//----------------------------------------------------------------------------
/*
 * File: btn.c
 * - service for testing of buttons
 * 
 * Part of Emulation of PMD 32 drive.
 * Based on MMC/SD/SDHC card library by Roland Riegel
 *   - http://www.roland-riegel.de/sd-reader/
 *
 * Author: Roman Brik { pmd85emu (at) gmail.com }
 * Web: http://pmd85.borik.net
 * Date: 15 Sep 2009
 * Version 1.3
*/
//----------------------------------------------------------------------------
#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>

#include "timer.h"
#include "btn.h"
#include "lcd.h"
//----------------------------------------------------------------------------
static volatile uint8_t btn_code;
static volatile uint8_t btn_code_q;
//----------------------------------------------------------------------------
void btn_service(uint32_t t);
//----------------------------------------------------------------------------
void btn_init(void)
{
  BTN_PORT_CTRL &= BTN_PORT_DIR;  // set port pins as input
  BTN_PORT_DATA |= BTN_PORT_PUP;  // enable pul-ups for button pins

  btn_code = 0; // no button pressed
  btn_code_q = 0;

  timer_add_service(btn_service);
}
//----------------------------------------------------------------------------
uint8_t btn_test(void)
{
  return btn_code;  // return code of the currently pressed button
}
//----------------------------------------------------------------------------
uint8_t btn_read(void)
{
  while (btn_code == 0) // wait for button press
  {
    sleep_mode();
  }

  btn_code_q = btn_code;
  btn_code = 0;

  return btn_code_q;
}
//----------------------------------------------------------------------------
void btn_service(uint32_t t)
{
  static uint16_t bl_ticks = 0;
  static uint8_t ticks = 0;
  static uint8_t btn = 0;
  uint8_t p, b;

  p = BTN_PORT_PIN;
  if (btn_code_q != 0) // waiting for release of button
  {
    if ((p & BTN_ALL_UP) == BTN_ALL_UP)
    {
      btn_code_q = 0; // button was released
      btn = 0;
      bl_ticks = 0; // restart backlight off timer
    }
    return;
  }

  // backlight is automatically turned off
  // if no button is pressed some seconds
  if (lcd_is_backlight_on())
  {
    bl_ticks++;
    if (bl_ticks >= BTN_BL_OFF_DELAY)
      lcd_backlight_off();
  }

  // test for button press
  b = 0;
  if ((p & BTN_SET) == 0)
    b = BTN_SET;
  else if ((p & BTN_CANCEL) == 0)
    b = BTN_CANCEL;
  else if ((p & BTN_LEFT) == 0)
    b = BTN_LEFT;
  else if ((p & BTN_RIGHT) == 0)
    b = BTN_RIGHT;

  if (b != 0) // if any button is pressed
  {
    if (b == btn) // button was pressed before
    {
      // buttn must be hold at least BTN_MIN_DELAY ticks
      ticks++;
      if (ticks >= BTN_MIN_DELAY)
      {
        if (lcd_is_backlight_on()) // if backlight is on
        {
          btn_code = b;            // accept new button
          btn_code_q = 0;
        }
        else
        {
          lcd_backlight_on();     // turn backlight on
          btn_code_q = b;         // wait for button release
          btn_code = 0;           // this button press not accepted
        }
      }
    }
    else  // new button
    {
      btn_code = 0;
      btn_code_q = 0;
      btn = b;    // code of button
      ticks = 0;  // reset tick counter
    }
  }
}
//----------------------------------------------------------------------------
